@@ -32,14 +32,13 @@ package SNMP::Info::CiscoStack;
 
 use strict;
 use Exporter;
-use SNMP::Info;
 
-@SNMP::Info::CiscoStack::ISA       = qw/SNMP::Info Exporter/;
+@SNMP::Info::CiscoStack::ISA       = qw/Exporter/;
 @SNMP::Info::CiscoStack::EXPORT_OK = qw//;
 
 use vars qw/$VERSION %MIBS %FUNCS %GLOBALS %MUNGE %PORTSTAT/;
 
-$VERSION = '2.06';
+$VERSION = '3.23';
 
 %MIBS = ( 'CISCO-STACK-MIB' => 'ciscoStackMIB', );
 
@@ -65,6 +64,7 @@ $VERSION = '2.06';
     'm_type'         => 'moduleType',
     'm_model'        => 'moduleModel',
     'm_serial'       => 'moduleSerialNumber',
+    'm_serial_string' => 'moduleSerialNumberString',
     'm_status'       => 'moduleStatus',
     'm_name'         => 'moduleName',
     'm_ports'        => 'moduleNumPorts',
@@ -159,7 +159,7 @@ sub i_duplex {
         next if ( defined $partial and $iid !~ /^$partial$/ );
 
         # Test for gigabit
-        if ( $p_duplex_cap->{$port} && $p_duplex_cap->{$port} == 0 ) {
+        if ( defined $p_duplex_cap->{$port} and $p_duplex_cap->{$port} == 0 ) {
             $i_duplex->{$iid} = 'full';
         }
 
@@ -194,7 +194,7 @@ sub i_duplex_admin {
         next if ( defined $partial and $iid !~ /^$partial$/ );
 
         # Test for gigabit
-        if ( $p_duplex_cap->{$port} && $p_duplex_cap->{$port} == 1 ) {
+        if ( defined $p_duplex_cap->{$port} and $p_duplex_cap->{$port} == 1 ) {
             $i_duplex_admin->{$iid} = 'full';
         }
 
@@ -214,9 +214,8 @@ sub i_speed_admin {
     my $partial = shift;
 
     my %i_speed_admin;
-    my $p_port  = $stack->p_port() || {};
-    my %mapping = reverse %$p_port;
-    my $p_speed = $stack->p_speed( $mapping{$partial} );
+    my $p_port  = $stack->p_port()  || {};
+    my $p_speed = $stack->p_speed() || {};
 
     my %speeds = (
         'autoDetect'      => 'auto',
